//
//  ViewControllerTest.m
//  SimpleCalculator
//
//  Created by Peter Hauke on 02.10.13.
//  Copyright (c) 2013 2sox. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "ViewController.h"
#import "FakeTextContainer.h"
@interface TestViewController : XCTestCase
@property (strong) ViewController *viewController;

@property (strong) FakeTextContainer *firstNumberField;
@property (strong) FakeTextContainer *secondNumberField;
@property (strong) FakeTextContainer *resultLabel;
@end

@implementation TestViewController

- (void)setUp {
    [super setUp];

    // get viewController
    self.viewController = [[ViewController alloc] init];
    [self.viewController viewDidLoad];
    
    // get FakeText Containers
    self.firstNumberField = [[FakeTextContainer alloc] init];
    self.secondNumberField = [[FakeTextContainer alloc] init];
    self.resultLabel = [[FakeTextContainer alloc] init];
    
    // set Outlets to fake containers
    self.viewController.firstNumberField = (UITextField*)self.firstNumberField;
    self.viewController.secondNumberField = (UITextField*)self.secondNumberField;
    self.viewController.resultLabel = (UILabel*)self.resultLabel;
}

- (void)tearDown {
    // Put teardown code here; it will be run once, after the last test case.
    self.firstNumberField = nil;
    self.secondNumberField = nil;
    self.resultLabel = nil;
    self.viewController = nil;
    [super tearDown];
}

-(void)testDoAddInputValues {
    // validate UI input and result

    self.viewController.firstNumberField.text = @"5";
    self.viewController.secondNumberField.text = @"10";
    [self.viewController doAddInputValues];
    XCTAssertEqual(self.viewController.resultLabel.text.floatValue, 15.0f,
                   @"Should be 15!");
    
    self.viewController.firstNumberField.text = @"5";
    self.viewController.secondNumberField.text = @"";
    [self.viewController doAddInputValues];
    XCTAssertEqualObjects(self.viewController.resultLabel.text,
                          @"NaN",
                          @"Should be NaN!");
}

-(void)testTextFieldShouldReturn {
    // simple tests for textFieldShouldReturn
    BOOL returnFirstNumberField = [self.viewController textFieldShouldReturn:(UITextField *)self.viewController.firstNumberField];
    XCTAssertTrue(returnFirstNumberField,
                  @"textFieldShouldReturn firstNumberField should be TRUE");
    
    BOOL returnSecondNumberField = [self.viewController textFieldShouldReturn:(UITextField *)self.viewController.secondNumberField];
    XCTAssertTrue(returnSecondNumberField,
                  @"textFieldShouldReturn secondNumberField should be TRUE");
}

@end
