//
//  TicTacToeViewController_Tests.m
//  TicTacToe
//
//  Created by Ingo Kasprzak on 19.09.13.
//  Copyright (c) 2013 Silutions. All rights reserved.
//

@import XCTest;
#import "ViewController.h"

@interface TicTacToeViewController_Tests : XCTestCase

@property (strong) ViewController *controller;

@end


@implementation TicTacToeViewController_Tests

- (void)setUp {
    [super setUp];
    // Put setup code here; it will be run once, before the first test case.
	self.controller = [[ViewController alloc] initWithNibName:NSStringFromClass([ViewController class]) bundle:nil];
	[self.controller loadView];
}

- (void)tearDown {
    // Put teardown code here; it will be run once, after the last test case.
	self.controller = nil;
    [super tearDown];
}


- (void)testNumberOfFields {
	// Check, if number of game fields views is correct
	XCTAssertTrue(self.controller.fields.count==9,
				  @"Number of game fields should be 9, but it is %i", self.controller.fields.count);
}

- (void)testSpacesBetweenGameFields {
	// Test the UI specifications of the graphic designer

	CGFloat spacing = 1.0f; // Space between game fields should be 1
	
	NSArray *fields = self.controller.fields;
	
	// Sort game field views by tag
	fields = [fields sortedArrayUsingComparator:^NSComparisonResult(id obj1, id obj2) {
		if ([obj1 tag] > [obj2 tag]) { return NSOrderedDescending; }
		if ([obj1 tag] < [obj2 tag]) { return NSOrderedAscending; }
		return NSOrderedSame;
	}];
	
	// Test spaces between game field views
	for (NSInteger i=0; i<(fields.count-3); i++) {
		if (i%3<2) {
			UIButton *srcButton = fields[i];
			UIButton *rightButton = fields[i+1];
			UIButton *lowerButton = fields[i+3];
			
			CGFloat expectedXPos = srcButton.frame.origin.x+srcButton.frame.size.width+spacing;
			CGFloat expectedYPos = srcButton.frame.origin.y+srcButton.frame.size.height+spacing;
			
			XCTAssertTrue(expectedXPos==rightButton.frame.origin.x,
						   @"Horizontal space between game fields should be %.0f for field %i and %i, but it is %.0f!",
						   spacing, srcButton.tag, rightButton.tag,
						   rightButton.frame.origin.x+spacing-expectedXPos);
			XCTAssertTrue(expectedYPos==lowerButton.frame.origin.y,
						   @"Vertical space between game fields should be %.0f for field %i and %i, but it is %.0f!",
						   spacing, srcButton.tag, lowerButton.tag,
						   lowerButton.frame.origin.y+spacing-expectedYPos);
		}
	}
}

- (void)testInterfaceOrientations {
	// Test allowed interface orientations
	XCTAssertTrue([self.controller supportedInterfaceOrientations] & UIInterfaceOrientationMaskPortrait,
				  @"App should allow portrait orientation!");
	XCTAssertTrue([self.controller supportedInterfaceOrientations] & UIInterfaceOrientationMaskPortraitUpsideDown,
				  @"App should allow upside down orientation!");
	XCTAssertFalse([self.controller supportedInterfaceOrientations] &
				   (UIInterfaceOrientationMaskLandscapeLeft|UIInterfaceOrientationMaskLandscapeRight),
				  @"App should NOT allow any landscape orientation!");
}

@end
