//
//  TableViewController.m
//  TicTacToe
//
//  Created by Kasprzak Ingo on 03.07.12.
//  Copyright (c) 2012 Silutions. All rights reserved.
//

#import "TableViewController.h"
#import "Game+Methods.h"
#import "Move+Methods.h"
#import "TicTacToeCell.h"

@interface TableViewController ()
@property (strong) NSFetchedResultsController *resultsController;
@end

@implementation TableViewController
@synthesize resultsController;

- (id)initWithStyle:(UITableViewStyle)style {
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)configureCell:(TicTacToeCell*)theCell atIndexPath:(NSIndexPath*)indexPath {
	static __strong NSDateFormatter *dateFormatter = nil;
	if (!dateFormatter) {
		dateFormatter = [[NSDateFormatter alloc] init];
		[dateFormatter setDateStyle:NSDateFormatterMediumStyle];
		[dateFormatter setTimeStyle:NSDateFormatterShortStyle];
	}
    
	Game *game = [self.resultsController objectAtIndexPath:indexPath];
	theCell.label.text = [dateFormatter stringFromDate:game.creationDate];

	[theCell.fields enumerateObjectsUsingBlock:^(UILabel *label, NSUInteger idx, BOOL *stop) {
		label.text = @"";
	}];
	
	NSSet *moves = game.moves;
	[moves enumerateObjectsUsingBlock:^(Move *obj, BOOL *stop) {
		UILabel *field = [[theCell.fields filteredArrayUsingPredicate:
						   [NSPredicate predicateWithFormat:@"tag=%@", obj.field]]
							lastObject];

		NSSet *winMoves = [game winningCombo];
		UIColor *color = (winMoves && [winMoves containsObject:obj]) ? [UIColor redColor] : [UIColor darkTextColor];
		field.textColor = color;
		field.text = ([obj.player intValue] ? @"O" : @"X");
	}];
}

- (void)viewDidLoad {
    [super viewDidLoad];
	
	TicTacToeDatabase *database = [TicTacToeDatabase sharedTicTacToeDatabase];
	NSFetchRequest *fetch = [NSFetchRequest fetchRequestWithEntityName:NSStringFromClass([Game class])];
	fetch.predicate = [NSPredicate predicateWithFormat:@"ANY moves.uid != NIL"];
	fetch.sortDescriptors = @[[NSSortDescriptor sortDescriptorWithKey:ManagedObjectCreationDateKey
															ascending:NO]];
	
	self.resultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetch
																 managedObjectContext:database.managedObjectContext
																   sectionNameKeyPath:nil
																			cacheName:@"TicTacToeCache"];
	self.resultsController.delegate = self;
	[self.resultsController performFetch:nil];

    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

- (void)viewDidUnload {
	[self setResultsController:nil];

    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	id <NSFetchedResultsSectionInfo> sectionInfo = [[self.resultsController sections] objectAtIndex:section];
    return [sectionInfo numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"TicTacToeCell";
    TicTacToeCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
	if (!cell) {
		NSArray *topLevelObjects = [[NSBundle mainBundle] loadNibNamed:NSStringFromClass([TicTacToeCell class])
																 owner:nil options:nil];
		cell = [topLevelObjects lastObject];
	}
	
	[self configureCell:cell atIndexPath:indexPath];
    
    return cell;
}

// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return [self.resultsController objectAtIndexPath:indexPath] != [[TicTacToeDatabase sharedTicTacToeDatabase] activeGame];
}

- (void) tableView:(UITableView *)tableView
commitEditingStyle:(UITableViewCellEditingStyle)editingStyle
 forRowAtIndexPath:(NSIndexPath *)indexPath {

    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
		Game *game = [self.resultsController objectAtIndexPath:indexPath];
		[[[TicTacToeDatabase sharedTicTacToeDatabase] managedObjectContext] deleteObject:game];
		[[TicTacToeDatabase sharedTicTacToeDatabase] saveChanges];
    }
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
    }   
}

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     */
}

#pragma mark - Fetched results Controller delegate

/*
 Assume self has a property 'tableView' -- as is the case for an instance of a UITableViewController
 subclass -- and a method configureCell:atIndexPath: which updates the contents of a given cell
 with information from a managed object at the given index path in the fetched results controller.
 */

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView beginUpdates];
}


- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
		   atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type {
	
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex]
						  withRowAnimation:UITableViewRowAnimationFade];
            break;
			
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex]
						  withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}


- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
	   atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
	  newIndexPath:(NSIndexPath *)newIndexPath {
	
    UITableView *tableView = self.tableView;
	
    switch(type) {
			
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:@[newIndexPath]
							 withRowAnimation:UITableViewRowAnimationFade];
            break;
			
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:@[indexPath]
							 withRowAnimation:UITableViewRowAnimationFade];
            break;
			
        case NSFetchedResultsChangeUpdate:
            [self configureCell:(TicTacToeCell*)[tableView cellForRowAtIndexPath:indexPath]
					atIndexPath:indexPath];
            break;
			
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:@[indexPath]
							 withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:@[newIndexPath]
							 withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}


- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView endUpdates];
}

@end
